@@ -1,12 +1,8 @@
 #include "maxmind.h"
 #include <stdarg.h>
 
-static inline std::string mmdb_convert_string(const MMDB_entry_data_s &entry){
-  std::string output(entry.utf8_string, 0, entry.data_size);
-  return output;
-}
 
-static std::vector < std::string > mmdb_getval(MMDB_s *data, const std::vector< std::string > &ip_addresses, ...){
+static std::vector <std::string> mmdb_getstring(MMDB_s *data, const std::vector< std::string > &ip_addresses, ...){
   int input_size = ip_addresses.size();
   std::vector < std::string > output(input_size);
   
@@ -33,7 +29,44 @@ static std::vector < std::string > mmdb_getval(MMDB_s *data, const std::vector<
       if((run_status != MMDB_SUCCESS) | (!entry_data.has_data)){
         output[i] ="Unknown";
       } else {
-        output[i] = mmdb_convert_string(entry_data);
+        std::string out(entry_data.utf8_string, 0, entry_data.data_size);
+        output[i] = out;
+      }
+    }
+  }
+  
+  return output;
+}
+
+
+static std::vector <double> mmdb_getdouble(MMDB_s *data, const std::vector< std::string > &ip_addresses, ...){
+  int input_size = ip_addresses.size();
+  std::vector < double > output(input_size);
+  
+  int run_status;
+  int lookup_error;
+  int gai_error;
+  MMDB_lookup_result_s result;
+  MMDB_entry_data_s entry_data;
+  va_list path;
+  
+  
+  for(int i = 0; i < input_size; i++){
+    if((i % 10000) == 0){
+      Rcpp::checkUserInterrupt();
+    }
+    
+    result = MMDB_lookup_string(data, ip_addresses[i].c_str(), &lookup_error, &gai_error);
+    if((lookup_error != MMDB_SUCCESS) | (gai_error != MMDB_SUCCESS)){
+      output[i] = NA_REAL;
+    } else {
+      va_start(path, ip_addresses);
+      run_status = MMDB_vget_value(&result.entry, &entry_data, path);
+      va_end(path);
+      if((run_status != MMDB_SUCCESS) | (!entry_data.has_data)){
+        output[i] = NA_REAL;
+      } else {
+        output[i] = entry_data.double_value;
       }
     }
   }
@@ -43,31 +76,39 @@ static std::vector < std::string > mmdb_getval(MMDB_s *data, const std::vector<
 
 
 std::vector < std::string > maxmind_bindings::continent_name(MMDB_s *data, std::vector < std::string >& ip_addresses){
-  return mmdb_getval(data, ip_addresses, "continent", "names", "en", NULL);
+  return mmdb_getstring(data, ip_addresses, "continent", "names", "en", NULL);
 }
 
 std::vector < std::string > maxmind_bindings::country_name(MMDB_s *data, std::vector < std::string >& ip_addresses){
-  return mmdb_getval(data, ip_addresses, "country", "names", "en", NULL);
+  return mmdb_getstring(data, ip_addresses, "country", "names", "en", NULL);
 }
 
 std::vector < std::string > maxmind_bindings::country_code(MMDB_s *data, std::vector < std::string >& ip_addresses){
-  return mmdb_getval(data, ip_addresses, "country", "iso_code", NULL);
+  return mmdb_getstring(data, ip_addresses, "country", "iso_code", NULL);
 }
 
 std::vector < std::string > maxmind_bindings::region_name(MMDB_s *data, std::vector < std::string >& ip_addresses){
-  return mmdb_getval(data, ip_addresses, "subdivisions", "0", "names", "en", NULL);
+  return mmdb_getstring(data, ip_addresses, "subdivisions", "0", "names", "en", NULL);
 }
 
 std::vector < std::string > maxmind_bindings::city_name(MMDB_s *data, std::vector < std::string >& ip_addresses){
-  return mmdb_getval(data, ip_addresses,"city", "names", "en", NULL);
+  return mmdb_getstring(data, ip_addresses,"city", "names", "en", NULL);
 }
 
 std::vector < std::string > maxmind_bindings::timezone(MMDB_s *data, std::vector < std::string >& ip_addresses){
-  return mmdb_getval(data, ip_addresses, "location", "time_zone", NULL);
+  return mmdb_getstring(data, ip_addresses, "location", "time_zone", NULL);
 }
 
 std::vector < std::string > maxmind_bindings::connection(MMDB_s *data, std::vector < std::string >& ip_addresses){
-  return mmdb_getval(data, ip_addresses, "connection_type", NULL);
+  return mmdb_getstring(data, ip_addresses, "connection_type", NULL);
+}
+
+std::vector < double > maxmind_bindings::latitude(MMDB_s *data, std::vector < std::string >& ip_addresses){
+  return mmdb_getdouble(data, ip_addresses, "location", "latitude", NULL);
+}
+
+std::vector < double > maxmind_bindings::longitude(MMDB_s *data, std::vector < std::string >& ip_addresses){
+  return mmdb_getdouble(data, ip_addresses, "location", "longitude", NULL);
 }
 
 List maxmind_bindings::lookup(std::vector < std::string >& ip_addresses, MMDB_s *mmdb_set,
@@ -89,6 +130,10 @@ List maxmind_bindings::lookup(std::vector < std::string >& ip_addresses, MMDB_s
       output.push_back(city_name(mmdb_set, ip_addresses));
     } else if(fields[i] == "timezone"){
       output.push_back(timezone(mmdb_set, ip_addresses));
+    } else if(fields[i] == "latitude"){
+      output.push_back(latitude(mmdb_set, ip_addresses));
+    } else if(fields[i] == "longitude"){
+      output.push_back(longitude(mmdb_set, ip_addresses));
     } else if(fields[i] == "connection"){
       output.push_back(connection(mmdb_set, ip_addresses));
     }
